/*
Copyright (c) 2009, INRIA, Université de Nancy 2 and Universidade
Federal do Rio Grande do Norte.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   * Neither the name of the Université de Nancy 2 or the Universidade Federal
     do Rio Grande do Norte nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.

THIS SOFTWARE IS PROVIDED BY INRIA, Université de Nancy 2 and
Universidade Federal do Rio Grande do Norte ''AS IS'' AND ANY EXPRESS
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL INRIA, Université de Nancy 2 and
Universidade Federal do Rio Grande do Norte BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/

/**
  \file veriT-signal.c

  \author David Deharbe

  \brief Implementation of signal handling functionality.

*/

#include <signal.h>
#ifdef PEDANTIC
#include <stdio.h>
#endif

#include "general.h"
#include "table.h"


#include "veriT-signal.h"

#ifndef	NSIG
#ifdef	_NSIG
#define	NSIG	_NSIG
#else
#define	NSIG	32
#endif
#endif

/* #define CAPTURE_SIGNALS
   PF: there are several problems to solve here:
   - signal is not portable.  One should use sigaction.
   - one cannot put a handler on SIGSTOP and SIGKILL, and thus I think
   this is not the right way to clear the system from eprover processes
   - I had plenty of problems on grid5k because of SIGSTOP signals while
   profiling.  I think there were generated by the profiler / not sure of it
*/

/*--------------------------------------------------------------*/

#ifdef CAPTURE_SIGNALS
static Ttable veriT_signal_table;
#endif

/*--------------------------------------------------------------*/

#ifdef CAPTURE_SIGNALS
static void veriT_signal_handle(int sig)
{
  unsigned i;
  for (i = 0; i < table_length(veriT_signal_table); i += 2)
    {
      Tsig_handle f = (Tsig_handle) table_get(veriT_signal_table, i);
      void * P = table_get(veriT_signal_table, i+1);
      (*f)(sig, P);
    }
}
#endif

/*--------------------------------------------------------------*/

void veriT_signal_init(void)
{
#ifdef CAPTURE_SIGNALS
  unsigned i;
  veriT_signal_table = table_new(32, 8);
  for (i = 0; i < NSIG; ++i)
    signal(i, &veriT_signal_handle);
#endif
}

/*--------------------------------------------------------------*/

void veriT_signal_done(void)
{
#ifdef CAPTURE_SIGNALS
  unsigned i;
  for (i = 0; i < NSIG; ++i)
    signal(i, SIG_DFL);
  table_free(&veriT_signal_table);
#endif
}

/*--------------------------------------------------------------*/

void veriT_signal_register(Tsig_handle sig_handle, void * P)
{
#ifdef CAPTURE_SIGNALS
  table_push(veriT_signal_table, sig_handle);
  table_push(veriT_signal_table, P);
#else
#ifdef PEDANTIC
  /* PF just to make sure arguments are used, so that no warning */
  if (sig_handle)
    printf("%p\n", P);
#endif
#endif
}

